#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Message::Storage::Mongo class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
# Comments:
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 4380;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Message::Storage::Mongo';

        use_ok('Tester');
        use_ok('Mentat::Handyman');
        use_ok('Mentat::Storage::Mongo');
        use_ok('Mentat::Message::Factory');
        use_ok('Value::Convertor');
        use_ok('Mentat::Message::Convertor::Mongo');
        use_ok('Mentat::Message::Convertor::IDEA');
        use_ok('Mentat::Message::Builder::IDEA');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
#### $Mentat::Message::Storage::Mongo::VERSION

#-------------------------------------------------------------------------------

my $CNT = 0;
sub gen_msg_rules()
{
    ++$CNT;
    my $message_rules = [
        ['EventTime','%ts:$detect_time'],
        ['ConnCount','$5'],
        ['Node[1]/SW','Kippo'],
        ['Node[1]/Tags[1]','Network'],
        ['Node[1]/Tags[2]','Honeypot'],
        ['Node[1]/Tags[3]','Kippo'],
        ['Source[1]/%ipt:$1[1]','$1'],
        ['Target[1]/%ipt:$2[1]','$2'],
        ['Target[1]/Port[1]','22'],
        ['Category[1]','Intrusion'],
        ['Completion','$4'],
        ['Description','User $3 $4 login from $1'],
        ['Note','Remote login via SSH'],
    ];

    return $message_rules;
}

#-------------------------------------------------------------------------------

# Prepare builder configurations
my $BUILDER_CONFIG = {
        'name' => 'tld.domain.node',
        'type' => 'kippo,honeypot',
        'sw'   => 'kippo',
        'note' => 'Some note',
    };

# Prepare message configurations
my $message_config = {
        'detect_time' => time,
        '1' => '127.0.0.1',
        '2' => '127.0.0.2',
        '3' => 'root',
        '4' => 'failed',
        '5' => 10,
    };

# Now attempt to create a class instance and check it
my $builder = Mentat::Message::Builder::IDEA->new($BUILDER_CONFIG);
isa_ok($builder, 'Mentat::Message::Builder::IDEA');

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new(database => 'mentat', collection => 'unittest_alerts');
isa_ok($instance, $TESTED_CLASS);

my ($tcount, $message, $result, $count);
eval {
    $instance->create_indexes();

    # Count current number of records in database
    $tcount = $instance->count();
    #### Number of records currently in database: $tcount

    # Check storing to database
    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id1 = $instance->insert_msg($message);
    isnt($record_id1, undef, 'Insert test 01');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id2 = $instance->insert_msg($message);
    isnt($record_id2, undef, 'Insert test 02');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id3 = $instance->insert_msg($message);
    isnt($record_id3, undef, 'Insert test 03');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id4 = $instance->insert_msg($message);
    isnt($record_id4, undef, 'Insert test 04');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id5 = $instance->insert_msg($message);
    isnt($record_id5, undef, 'Insert test 05');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id6 = $instance->insert_msg($message);
    isnt($record_id6, undef, 'Insert test 06');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id7 = $instance->insert_msg($message);
    isnt($record_id7, undef, 'Insert test 07');

    $message = $builder->build(rules     => gen_msg_rules(),
                               name_args => $message_config);
    isa_ok($message, 'Mentat::Message::IDEA');
    my $record_id8 = $instance->insert_msg($message);
    isnt($record_id8, undef, 'Insert test 08');

    is($instance->count(), $tcount + 8, 'Count test 01');

    # Retrieve some values, that are converted for database (for future back check)
    my $record_id8_ip = $message->path_value('Source/IP4');
    my $record_id8_ts = $message->path_value('CreateTime');

    # Check fetching from database
    isa_ok($instance->fetch_msg($record_id1), 'Mentat::Message');
    isa_ok($instance->fetch_msg($record_id2), 'Mentat::Message');
    isa_ok($instance->fetch_msg($record_id3), 'Mentat::Message');
    isa_ok($instance->fetch_msg($record_id4), 'Mentat::Message');
    isa_ok($instance->fetch_msg($record_id5), 'Mentat::Message');
    isa_ok($instance->fetch_msg($record_id6), 'Mentat::Message');
    isa_ok($instance->fetch_msg($record_id7), 'Mentat::Message');
    isa_ok($instance->fetch_msg($record_id8), 'Mentat::Message');
    is($instance->fetch_msg('foobar'), undef, 'Fetch test 09');

    # Check deleting from database
    my $dr1 = $instance->delete($record_id1);
    my $dr2 = $instance->delete($record_id2);
    my $dr3 = $instance->delete('bogus__id');
    is_deeply([$dr1->{'ok'}, $dr1->{'n'}, $dr1->{'err'}], ['1', 1, undef], "Delete test 01");
    is_deeply([$dr2->{'ok'}, $dr2->{'n'}, $dr2->{'err'}], ['1', 1, undef], "Delete test 02");
    is_deeply([$dr3->{'ok'}, $dr3->{'n'}, $dr3->{'err'}], ['1', 0, undef], "Delete test 03");

    # Check counting from database
    is($instance->count({'_id' => $record_id1}), 0, 'Count test 01');
    is($instance->get_last_filter(), '[COUNT] Database: \'mentat:unittest_alerts\', filter \'{"_id":"'.$record_id1.'"}\'', 'Last filter test 01');
    is($instance->count({'_id' => $record_id2}), 0, 'Count test 02');
    is($instance->get_last_filter(), '[COUNT] Database: \'mentat:unittest_alerts\', filter \'{"_id":"'.$record_id2.'"}\'', 'Last filter test 02');
    is($instance->count({'_id' => $record_id3}), 1, 'Count test 03');
    is($instance->get_last_filter(), '[COUNT] Database: \'mentat:unittest_alerts\', filter \'{"_id":"'.$record_id3.'"}\'', 'Last filter test 03');
    is($instance->count({'_id' => $record_id4}), 1, 'Count test 04');
    is($instance->get_last_filter(), '[COUNT] Database: \'mentat:unittest_alerts\', filter \'{"_id":"'.$record_id4.'"}\'', 'Last filter test 04');
    is($instance->count(), $tcount + 6, 'Count test 05');
    is($instance->get_last_filter(), '[COUNT] Database: \'mentat:unittest_alerts\', filter \'\'', 'Last filter test 05');

    # Check searching from database
    ($result, $count) = $instance->find_msg();
    is($count, 6, 'Find test A01');
    is(scalar @$result, 6, 'Find test B01');
    isa_ok($result->[0], 'Mentat::Message');
    is($instance->get_last_filter(), '[FIND] Database: \'mentat:unittest_alerts\', filter \'\' with limit \'500000\'', 'Last filter test 06');

    ($result, $count) = $instance->find_msg({'_id' => $record_id3});
    is($count, 1, 'Find test A02');
    is(scalar @$result, 1, 'Find test B02');
    isa_ok($result->[0], 'Mentat::Message');
    is($instance->get_last_filter(), '[FIND] Database: \'mentat:unittest_alerts\', filter \'{"_id":"'.$record_id3.'"}\' with limit \'500000\'', 'Last filter test 07');

    ($result, $count) = $instance->find_msg_wr();
    is($count, 6, 'Find test A07');
    is(scalar @$result, 6, 'Find test B07');
    is(ref $result->[0]->[1], 'HASH', 'Find test C07');
    isa_ok($result->[0]->[0], 'Mentat::Message');
    is($instance->get_last_filter(), '[FIND] Database: \'mentat:unittest_alerts\', filter \'\' with limit \'500000\'', 'Last filter test 12');

    ($result, $count) = $instance->find_msg_wr({'_id' => $record_id3});
    is($count, 1, 'Find test A08');
    is(scalar @$result, 1, 'Find test B08');
    is(ref $result->[0]->[1], 'HASH', 'Find test C08');
    isa_ok($result->[0]->[0], 'Mentat::Message');
    is($instance->get_last_filter(), '[FIND] Database: \'mentat:unittest_alerts\', filter \'{"_id":"'.$record_id3.'"}\' with limit \'500000\'', 'Last filter test 13');

    my ($rt1, $cnt) = $instance->find_msg_wr({'_id' => $record_id8});
    #diag(Dumper($rt1));
    my ($date, $ip) = ($rt1->[0]->[1]->{ts}, $rt1->[0]->[1]->{Source}->[0]->{IP4}->[0]->{ip});

    $date = Value::Convertor->nbi_to_datestr(Value::Convertor->bin_to_bigint($date));
    $ip = Value::Convertor->bin_to_ipv4str($ip);
    is($date, $record_id8_ts, "Check test 01");
    is($ip,   $record_id8_ip, "Check test 02");

    # Try removing item from database
    my $rr1 = $instance->remove({'_id' => $record_id3});
    is_deeply([$rr1->{'ok'}, $rr1->{'n'}, $rr1->{'err'}], ['1', 1, undef], "Remove test 01");
    is($instance->get_last_filter(), '[REMOVE] Database: \'mentat:unittest_alerts\', filter \'{"_id":"'.$record_id3.'"}\'', 'Last filter test 18');

    is($instance->count({'_id' => $record_id3}), 0,           'Count test 05');
    is($instance->count({'_id' => $record_id4}), 1,           'Count test 06');
    is($instance->count(),                       $tcount + 5, 'Count test 07');

    # Try removing all items from database
    my $rr3 = $instance->remove();
    is_deeply([$rr3->{'ok'}, $rr3->{'n'}, $rr3->{'err'}], ['1', $tcount + 5, undef], "Remove test 02");
    is($instance->get_last_filter(), '[REMOVE] Database: \'mentat:unittest_alerts\', filter \'\'', 'Last filter test 22');
    is($instance->count(), 0, 'Count test 08');
};
if($@) {
    print STDERR "MongoDB error: $@\n";
    $instance->remove();
}

#-------------------------------------------------------------------------------
# Prepare handyman for creating messages
my $CNF_DIR = '../../conf';
$CNF_DIR = '/etc/mentat' unless -d $CNF_DIR;
my $TMP_DIR = '../../spool';
$TMP_DIR = '/tmp' unless -d $TMP_DIR;
my $handyman = Mentat::Handyman->new(template_file        => $CNF_DIR.'/msgtmplts.json',
                                     temp_folder          => $TMP_DIR,
                                     pickup_folder        => $TMP_DIR,
                                     cache_folder         => $TMP_DIR,
                                     #schema_file          => $CNF_DIR.'/schema-idea.json',
                                     template_schema_file => $CNF_DIR.'/schema-msgtmplts.json',
                                     user                 => 'mentat',
                                     builder_config       => $BUILDER_CONFIG,
                                     strict               => 1);
isa_ok($handyman, 'Mentat::Handyman');
#-------------------------------------------------------------------------------

my @tlist = $handyman->templates();
#my @tlist = ('n6-001', 'n6-002');
my ($idea, $template, @msg_args, $rid, $mhash, $cidea, $midea, $pv1, $pv2);
my $hconvertor = Mentat::Message::Convertor::IDEA->new();
foreach my $tid (@tlist) {
    eval {
        diag("TEMPLATE: $tid") if Tester::D();

        # Generate the test message
        $template = $handyman->template($tid);
        @msg_args = $handyman->parse_args($template->{'cmd_example'});
        $idea    = $handyman->generate($tid, {'detect_time' => time}, @msg_args);
        isa_ok($idea, 'Mentat::Message::IDEA');
        diag("IDEA message before insertion: ".$idea->to_string(1)) if Tester::D(2);

        $rid = $instance->insert_msg($idea);
        isnt($rid, undef, "Insert test for '$tid': '_id': '$rid'");

        $midea = $instance->fetch_msg($rid);
        isnt($midea, undef, "Fetch msg test for '$tid': '_id': '$rid'");
        diag("IDEA message after fetch: ".Dumper($midea)) if Tester::D(2);
        
        $mhash = $instance->fetch($rid);
        isnt($mhash, undef, "Fetch hash test for '$tid': '_id': '$rid'");
        diag("Mongo hash after fetch: ".Dumper($mhash)) if Tester::D(2);

        $cidea = $hconvertor->convert($mhash, 'mongo', '(Source|Target)\[\d+\]/IP(4|6)\[\d+\]');
        isnt($cidea, undef, "Conv   test for '$tid': '_id': '$rid'");
        diag("IDEA message after conversion: ".$cidea->to_string(1)) if Tester::D(2);

        my ($src_mpath, $value) = $idea->iterate();
        while ($src_mpath) {
            $pv1 = $midea->path_value($src_mpath);
            is("$value", "$pv1", "ORIG - FETCH comparison test for '$tid': '_id': '$rid': 'mpath': '$src_mpath' => '$value' != '$pv1'");

            $pv2 = $cidea->path_value($src_mpath);
            is("$value", "$pv2", "ORIG - CONV  comparison test for '$tid': '_id': '$rid': 'mpath': '$src_mpath' => '$value' != '$pv2'");

            ($src_mpath, $value) = $idea->iterate();
        }
    };
    if ($@) {
        next unless $tid;
        fail("Message storing test for template '$tid': $@");
    }
}

my $message_rules_special = [
    ['EventTime','2016-01-14 16:08:56Z'],
    ['ConnCount','20'],
    ['Node[1]/SW','Kippo'],
    ['Node[1]/Tags[1]','Network'],
    ['Node[1]/Tags[2]','Honeypot'],
    ['Node[1]/Tags[3]','Kippo'],
    ['Source[1]/IP4[1]','127.0.0.1'],
    ['Target[1]/IP4[1]','127.0.0.1/24'],
    ['Target[1]/Port[1]','22'],
    ['Category[1]','Intrusion'],
    ['Completion','completed'],
    ['Description','User login from somewhere'],
    ['Note','Remote login via SSH'],
];

$idea = $builder->build(rules     => $message_rules_special,
                        name_args => {});
isa_ok($message, 'Mentat::Message::IDEA');
$rid = $instance->insert_msg($idea);
isnt($rid, undef, "Special insert test for '_id': '$rid'");

$midea = $instance->fetch_msg($rid);
isnt($midea, undef, "Special fetch msg test for '_id': '$rid'");
diag("IDEA message after fetch: ".Dumper($midea)) if Tester::D(2);

$mhash = $instance->fetch($rid);
isnt($mhash, undef, "Special fetch hash test for '_id': '$rid'");
diag("Mongo hash after fetch: ".Dumper($mhash)) if Tester::D(2);

$cidea = $hconvertor->convert($mhash, 'mongo', '(Source|Target)\[\d+\]/IP(4|6)\[\d+\]');
isnt($cidea, undef, "Special conv test for '_id': '$rid'");
diag("IDEA message after conversion: ".$cidea->to_string(1)) if Tester::D(2);

my ($src_mpath, $value) = $idea->iterate();
while ($src_mpath) {
    $pv1 = $midea->path_value($src_mpath);
    is("$value", "$pv1", "ORIG - FETCH comparison test for '_id': '$rid': 'mpath': '$src_mpath' => '$value' != '$pv1'");

    $pv2 = $cidea->path_value($src_mpath);
    is("$value", "$pv2", "ORIG - CONV  comparison test for '_id': '$rid': 'mpath': '$src_mpath' => '$value' != '$pv2'");

    ($src_mpath, $value) = $idea->iterate();
}
    
my $rrf = $instance->remove();
is($rrf->{'n'}, 90, "Remove test final");
