#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Reporter module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use JSON;

use Test::More tests => 2;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Reporter';

        use_ok('Tester');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
Tester::DEBUG_ON();

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Mentat::Reporter::VERSION) if Tester::D();

### Attempting to create Mentat::Reporter::Engine::Default instance...
my $instance = $TESTED_CLASS->new(filter_grammar_file => '../../conf/filter-grammar.conf',
                                  filter_schema_file  => '../../conf/schema-data-filter.json',
                                  filter_config       => '../../rules/filter/mentat-reporter-filter.json',
                                  template_dir        => '../../formater-templates',
                                  temp_folder         => './spool',
                                  charts_dir          => './spool',
                                  reports_dir         => './spool',
                                  event_database      => 'mentat',
                                  event_collection    => 'events',
                                  mail_from           => 'test@mentat.cesnet.cz',
                                  reply_to            => 'jan.mach@cesnet.cz',
                                  mail_admin          => 'jan.mach@cesnet.cz',
                                  mail_devel          => 'jan.mach@cesnet.cz',
                                  test_mode           => 1,
                                  template_map_e      => {},
                                  template_map_a      => {
                                        'LaBrea' => 'labrea',
                                        'Kippo'  => {
                                                'Remote login' => 'kippo-001',
                                            },
                                    },
                                  dict => {
                                        'report_extra'   => 'Extra report',
                                        'report_summary' => 'Souhrnný report',
                                        'label_e'        => 'Jiný typ útoku/problému/události',
                                        'label_s'        => 'Jiné typy útoků/problémů/událostí',
                                    },
                                  pilot_mode => {
                                        'abuse@muni.cz' => 'rt@csirt-rt-devel.ics.muni.cz ',
                                        'abuse@zcu.cz'  => 'abuse@zcu.cz',
                                        'abuse@vsb.cz'  => 'abuse@vsb.cz',
                                    },
                                  simulate => 1,
                                  threshold_cache_db  => 'mentat',
                                  threshold_cache_col => 'test_cache_threshold',
                                );
#isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

my ($analysis, $reports_legacy, $reports_ng) = $instance->compare(start => '2015-05-20 20:01:00', stop => '2015-05-21 00:01:00');
#diag("RESULT:" . Dumper($analysis, $reports_legacy, $reports_ng));
diag("ANALYSIS:" . Dumper($analysis));

